function [alphas,means,mcovs]=mixturefit(data,ncomp,constraint,eps,means0)
% @ version 1.0.G
%
% [alphas,means,cov]=mixturefit(data,ncomp,constraint,eps)
%
% data is assumed to be ROW_WISE
% ncomp - number of components in mixture
% constraint - 0 for full covariance, 1 for diagonal covariance, 2 for spherical covariance
% eps - value of regularizer on posteriors and on covariance -- try 1e-8.
%
% mixturefit returns:
%
% alphas - the vector of mixing coefficients (cluster priors)
% means - means of the mixture model components -- columnwise by component.
% cov - array of covariance matrices

% This function calls pnormal (below) which calculates the probabilities p( x | i ).

%
% Define arrays dimensions:
%

[ndata,dim]=size(data);

alphas = 1/ncomp * ones(ncomp,1);    % Mixture coefficients.
means = zeros(dim,ncomp);            % ncomp COLUMN vectors.
dcovs = zeros(dim,dim,ncomp);        % Weighted data covariance matrices.
mcovs = zeros(dim,dim,ncomp);        % Model covariance matrices
pix = 1/(ncomp*ndata)*ones(ndata,ncomp);     % posteriors p( cluster i | datum x ).
% pix(j,k)=p(cluster k | x_j)
pnormalvals = zeros(ndata,ncomp);        % likelihoods pnormalvals(i,j)=p(x_i | cluster j)

% Initialize values in means and covs
% set means to ncomp randomly selected data vectors


if dim == 1
    thresholdx = linspace(min(data),max(data),ncomp+2);
    means = thresholdx(2:end-1);
else
    u = randperm(ndata);
    means = data( u(1:ncomp),: )';
end
% means=means0;
% set covariance matrices to identity
for i=1:ncomp
    mcovs(:,:,i) = diag(ones(1,dim));
end;

L = -1.0e20;
oldL = -2.0e20;
cnt = 0;

while ( (abs(L-oldL) > 0.001) & (cnt < 500) )

    cnt
    oldL = L;

    % Do EM
    %
    % E-step, estimate posteriors
    %

    % Calculate p(x|i)
    pnormalvals = pnormal(data,means,mcovs,ncomp);

    pix = (ones(ndata,ncomp)*diag(alphas)) .* pnormalvals;
    norm = sum(pix,2)*ones(1,ncomp);
    % edited for warning divide by zero

    norm(find(norm < eps)) = eps;

    pix = pix ./ norm;
    % Regularize posteriors
    pix = ( pix + (eps * ones(ndata,ncomp)) ) / (1+eps*ncomp);

    %
    % M-step
    %

    % First the weighted means
    for comp=1:ncomp
        weightdata = (pix(:,comp)*ones(1,dim)).*data;
        means(:,comp) = sum( weightdata )' / sum( pix(:,comp) );
    end;

    % Next the data cov. matrices
    for comp=1:ncomp
        shiftdata = data - ones(size(data)) * diag(means(:,comp));
        weightdata = (pix(:,comp)*ones(1,dim)) .* shiftdata;
        dcovs(:,:,comp) = ( weightdata' * shiftdata ) / sum( pix(:,comp) );
    end;

    if constraint==1
        % Build the model covariance for DIAGONAL case
        for comp=1:ncomp
            mcovs(:,:,comp) = diag(diag( dcovs(:,:,comp) ) );
        end;
    elseif constraint==2
        % Build the model covariance for SPHERICAL case
        for comp=1:ncomp
            mcovs(:,:,comp) = ( trace(dcovs(:,:,comp))/dim )*diag(ones(dim,1));
        end;
    else % if constraint==0 or anything else, model is UNCONSTRAINED
        for comp=1:ncomp
            mcovs(:,:,comp) = dcovs(:,:,comp);
        end;
    end;

    % Optimize the alphas
    alphas = sum(pix)'/ndata;

    % Compute and print the log likelihood per data point
    probs = pnormalvals * alphas;
    L = sum(log(probs)) / ndata;
    
    indzeros= probs==0;
    temp=probs;
    temp(indzeros)=[];
    
    L = sum(log(temp)) / ndata;
    cnt = cnt + 1;

end; % while





